////////////////////Utils
function quote(str) {
    str = String(str); // to make sure it is a string
    var c, i, l = str.length, o = "'";
    for (i = 0; i < l; i++) {
        c = str.charAt(i);
        if (c >= ' ') {
            if (c === '\\' || c === "'") {
                o += '\\';
            }
            o += c;
        } else {
            switch (c) {
                case '\b':
                    o += '\\b';
                    break;
                case '\f':
                    o += '\\f';
                    break;
                case '\n':
                    o += '\\n';
                    break;
                case '\r':
                    o += '\\r';
                    break;
                case '\t':
                    o += '\\t';
                    break;
                default:
                    c = c.charCodeAt();
                    o += '\\u00' + Math.floor(c / 16).toString(16) + (c % 16).toString(16);
            }
        }
    }
    return o + "'";
};

function esc(obj) {
	if (obj == null) return "";
    if (obj.replace) { // String 
    	//return Format.quote(obj);
    	return quote(obj);
    } else if (obj.getTime) { // Date Object
        obj = obj.getTime();
        return obj;                            
    } else { // number
    	return obj;
    }
}

function list(ids) {
	if (ids == null || ids.length == 0) return "()";
	var buf = ids.split('|');
	return "('" + buf.join("','") + "')";
}

////////////////// Date Utils
var DAY = 24*60*60*1000; // in milliseconds

// returns the current time in EPOCH time, in milliseconds
function now() {
	return new Date().getTime();
}

// returns today start, added by nbDays. at 00:00AM, in milliseconds
function today(nbDays) { 
	if (nbDays == null) nbDays = 0;
    var now = new Date(); //current date and time zone
    var todayStart = new Date(now.getFullYear(), now.getMonth(), now.getDate(), 0, 0, 0, 0)
    return todayStart.getTime() + nbDays*DAY;
}

// returns the day in epoch time, which is the first day of the current week at 00:00AM, and add nbWeeks to it
function week(nbWeeks) {
	if (nbWeeks == null) nbWeeks = 0;
    // get the number of days between now and the week start
    var dayOfWeek = new Date().getDay(); // returns the day of the week, 0 is Sunday
    return today(-dayOfWeek) + nbWeeks*7*DAY;
}

// returns the first day of the current month, added by addMonths in epoch time, at 00:00AM
function month(addMonths) {
	 if (addMonths == null) addMonths = 0;
	 var now = new Date(); //current date and time zone
	
	 var newYear = now.getFullYear();
	 var newMonth = now.getMonth() + addMonths;
	 var y_amount = parseInt(newMonth / 12);
	 if (y_amount > 0) {
	     newYear += y_amount;
	 }
	 newMonth = (newMonth % 12);
	 return new Date(newYear, newMonth, 1, 0, 0, 0, 0).getTime();
}

///////////////////////////// User class : new to API Level 6
//helper function returning the current username
function user() { 
	return User.getName(); 
}

function currency() {
	return User.getCurrency();
}

function User() {}

User.getCurrency = function() {
	var item = Query.selectId("System.settings", "currency");
	return (item != null) ? item.value : "USD";
}

User.getName = function() {
	// WARNING : it is ESSENTIAL to have the "" + before : it will force the conversion of the Object type return by Settigns.getUsername into a javascript string
	// and this is required with "{user}" extrapolation calling esc() then, quote();
	return "" + Settings.getUsername();
}

User.isAdmin = function() {
	var privilege = Settings.get("privilege");
	return (privilege == "1");
}

//////////////////////////
function Config () {}
Config.tables = new Array();

function R() {}

// Tablet friendly apps should redefine this function in their Code
// we define it here just in case the app does not redefine it, to avoid runtime script error
function leftPane() { List.show("leftpane"); }

//////////////////////////

function Query() {}

Query.insert = function(table, values) { return _Query.insert(table, values); }
Query.update = function(table, values, where) { _Query.update(table, values, where); }
Query.updateId = function(table, recordId, columnName, value) { _Query.updateId(table, recordId, columnName, value); }
Query.deleteId = function(table, recordId) { _Query.deleteId(table, recordId); }
Query.deleteWhere = function(table, where) { _Query.deleteWhere(table, where); }

Query.count = function(table, where) { return _Query.count(table, where); }
Query.options = function(table, where) { return _Query.options(table, where); }
Query.ids = function(table, where) { return _Query.ids(table, where); }

Query.select = function(table, columns, where, orderby) { return JSON.parse(_Query.select(table, columns, where, orderby)); }
Query.selectId = function(table, where) { return JSON.parse(_Query.selectId(table, where)); }
Query.rss = function(url) { return JSON.parse(_Query.rss(url)); }
Query.search = function(table, search, columns, where) { return JSON.parse(_Query.search(table, search, columns, where)); }

//////////////
// Custom fields
function CustomFields() {}

CustomFields.values = null; // Array of values indexed by name

CustomFields.view = function(table, recordId) {
	var item = Query.selectId(table, recordId);
	var fields = Query.select("Notes.fields", "name;label;type;seloptions", "formid=" + esc(table), "rank");
	if (fields.length == 0) return;
	
	CustomFields.loadValues(item.custom);
	
	List.addHeader(R.CUSTOMFIELDS);
	for (var i=0;i<fields.length;i++) {
		var field = fields[i];
		var value = CustomFields.values[field.name];
		if (value != null && value != "") {
			var onchange = "CustomFields._update(" +esc(table) + "," + esc(recordId) + ",this.id,this.value)";
			
			if (field.type=='select') {
				List.addComboBox(field.name, field.label, value, onchange, field.seloptions);
			} else if (field.type=='selectmulti') {
				List.addComboBoxMulti(field.name, field.label, value, onchange, field.seloptions);
			} else if (field.type=='checkbox') {
				List.addCheckBox(field.name, field.label, parseInt(value), onchange);
			} else if (field.type=='phonenumber') {
				List.addItemLabel(field.label, Format.phone(value), "App.call(this.value)");
			} else if (field.type=='email') {
				List.addItemLabel(field.label, value, "App.mailto(this.value)");
			} else if (field.type=='link') {
				List.addItemLabel(field.label, value, "App.web(this.value)");
			} else if (field.type=='contact') {
				var contact = Query.selectId("Contacts.contacts", value);
				if (contact != null) List.addItemLabel(field.label, contact.name, "Contacts.viewContact(" + esc(contact.id) + ")");
			} else if (field.type=='company') {
				var company = Query.selectId("Contacts.companies", value);
				if (company != null) List.addItemLabel(field.label, company.name, "Contacts.viewCompany(" + esc(company.id) + ")");
			} else {
				List.addTextBox(field.name, field.label, value, onchange, field.type);
			}
		}
	}
}

CustomFields.edit = function(table, recordId) {
	var item = Query.selectId(table, recordId);
	var fields = Query.select("Notes.fields", "name;label;type;seloptions", "formid=" + esc(table), "rank");
	if (fields.length == 0) return;
	
	CustomFields.loadValues(item.custom);
	
	List.addHeader(R.CUSTOMFIELDS);
	for (var i=0;i<fields.length;i++) {
		var field = fields[i];
		var value = CustomFields.values[field.name];
		var onchange = "CustomFields._update(" + esc(table) + "," + esc(recordId) + ",this.id,this.value)";
		if (field.type=='select') {
			List.addComboBox(field.name, field.label, value, onchange, field.seloptions);
		} else if (field.type=='selectmulti') {
			List.addComboBoxMulti(field.name, field.label, value, onchange, field.seloptions);
		} else if (field.type=='checkbox') {
			List.addCheckBox(field.name, field.label, parseInt(value), onchange);
		} else if (field.type=='contact') {
			List.addComboBoxMulti(field.name, field.label, value, onchange, Query.options("Contacts.contacts"));
		} else if (field.type=='company') {
			List.addComboBoxMulti(field.name, field.label, value, onchange, Query.options("Contacts.companies"));
		} else if (field.type=='project') {
			List.addComboBoxMulti(field.name, field.label, value, onchange, Query.options("Projects.projects", "status=0"));
		} else if (field.type=='user') {
			List.addComboBoxMulti(field.name, field.label, value, onchange, Query.options("System.users"));
		} else if (field.type == 'location') {
	        List.addGeoBox(field.name, field.label, value, onchange);
	    } else {
			List.addTextBox(field.name, field.label, value, onchange, field.type);
		}
	}
}

CustomFields.loadValues = function(custom) {
	CustomFields.values = null;
	try {
		CustomFields.values = JSON.parse(custom);
		if (CustomFields.values != null) {
			return;
		}
	} catch (e) {
		// Old serialization with pipe, continue
	}
	
	// Old code
	CustomFields.values = new Object();
	var values = custom.split("|");
	for (var i = 0; i < values.length; i++) {
		var parts = values[i].split(':');
		if (parts.length == 2) {
			var name = parts[0];
			var value = parts[1];
			CustomFields.values[name] = value;
		}
	}
}

CustomFields._update = function(table, recordId, name, value) {
	CustomFields.values[name] = value;
	var custom = JSON.stringify(CustomFields.values);
	Query.updateId(table, recordId, "custom", custom);
	
	/*
	var item = Query.selectId(table, recordId);
	var custom  = item.custom;
	var values = custom.split("|");
	var found = false;
	for (var i=0;i<values.length;i++) {
		if (values[i].indexOf(name+":") == 0) {
			values[i] = String(name + ":" + value);
			found = true;
			break;
		}
	}
	if (!found) {
		values.push(name + ":" + value);
	}
	custom = values.join("|");
	*/
	
	// This is essential : because the way we store the multivalue in one field.... we need to rebind the page after update
	//History.reload();
}

///// Edit Address 
function _editAddress(tableName, id) { Address.edit(tableName, id);}

function Address() {}

Address.edit = function(tableName, id) {
	var item = Query.selectId(tableName, id);
	var address = Format.address(item.street, item.city, item.zipcode, item.state, item.country);
	// we need History.reload, because address is passed as a parameter in List.addGeoBox
	var onchange = "Query.updateId(" + esc(tableName) + "," + esc(id) + ",this.id,this.value);History.reload()";
	
	Toolbar.setTitle(R.EDITADDRESS);
	Toolbar.setStyle("edit");
	List.addTextBox('street', R.STREET, item.street, onchange, 'text');
	List.addTextBox('city', R.CITY, item.city, onchange, 'text');
	List.addTextBox('state', R.STATE, item.state, onchange, 'text');
	List.addTextBox('zipcode', R.ZIPCODE, item.zipcode, onchange, 'text');
	List.addTextBox('country', R.COUNTRY, item.country, onchange, 'text');
	List.addHeader(R.GEOCOORDINATES);
	List.addGeoBox('mapurl', R.POSITION, item.mapurl, onchange, address);
    List.show();
}

//////////////////// Groupcount

function _groupcount(items, fieldName) {
    var map = new Array();
    for (var i = 0; i < items.length; i++) {
        var item = items[i];
        var value = item[fieldName];
        if (value != null) {
            var values = value.split('|');
            for (var j = 0; j < values.length; j++) {
                var count = map[values[j]];
                if (count = null) {
                    map[values[j]] = 1;
                } else {
                    map[values[j]] = count + 1;
                }
            }
        }
    }
    return map;
}

////////////////////////////

function List() { }

List._items = new Array();

// private method
List._addItem = function(type, label, value, onclick, style) {
	var item = new Object();
    item.type = type;
    if (label != null) item.label = String(label);
    if (value != null) item.value = String(value);
    if (onclick != null) item.onclick = String(onclick);
    if (style != null) item.style = String(style);
    List._items.push(item);
}

List._addEditItem = function(editStyle, id, label, value, onchange, options) {
	var item = new Object();
	item.editStyle = editStyle;
	if (id != null) item.id = String(id);
	if (label != null) item.label = String(label);
	if (value != null)  item.value = String(value);
	if (onchange != null)  item.onclick = String(onchange);
	if (options != null)  item.options = String(options);
	List._items.push(item);
}

List._removeLastItemIfHeader = function() {
	if (List._items.length > 0) {
		if (List._items[List._items.length-1].type == 3) {
			List._items.pop();
		}
	}
}

List._flush = function() {
	if (List._items.length > 0) {
		_List.addItems(JSON.stringify(List._items));
		List._items.length = 0;
	}
}

///////// Public Methods
List.addItem = function(text, onclick, style) {
	List._addItem(0, text, null, onclick, style);
}

List.addItemLabel = function(label, value, onclick, style) {
    if (value == null || value.length == 0) return;
	List._addItem(1, label, value, onclick, style);
}

List.addItemSubtitle = function(text, subtitle, onclick, style) {
	List._addItem(2, text, subtitle, onclick, style);
}

List.addHeader = function(title) {
	if (title == null) return;
	List._removeLastItemIfHeader();
	List._addItem(3, title);
}

List.addButton = function(text, onclick) {
	List._addItem(4, text, null, onclick);
}

List.addImage = function(imgurl, onclick) {
	var item = new Object();
    item.type = 5;
    if (imgurl != null) item.img = String(imgurl);
    if (onclick != null) item.onclick = String(onclick);
    List._items.push(item);
}

List.addItemRating = function(label, value, onclick, style) {
    List._addItem(6, label, value, onclick, style);
}

List.addItemColor = function(label, value, onclick, style) {
    List._addItem(7, label, value, onclick, style);
}

List.addTextBox = function(id, label, value, onchange, style) {
	if (style == null) style = "text";
	List._addEditItem(style, id, label, value, onchange);
}

List.addCheckBox = function(id, label, checked, onchange) {
	var value = checked ? "1" : "0";
	List._addEditItem("checkbox", id, label, value, onchange);
}

List.addComboBox = function (id, label, value, onchange, options) {
	List._addEditItem("combobox", id, label, value, onchange, options);
}

List.addComboBoxMulti = function(id, label, value, onchange, options) {
	List._addEditItem("comboboxmulti", id, label, value, onchange, options);
}

List.addSignatureBox = function(id, label, value, onchange) {
	List._addEditItem("signature", id, label, value, onchange);
}

List.addBarcodeBox = function(id, label, value, onchange) {
	List._addEditItem("barcode", id, label, value, onchange);
}

List.addGeoBox = function(id, label, value, onchange, address) {
	List._addEditItem("geo", id, label, value, onchange, address);
}

List.bindItems = function(table, column, where, onclick) {
	List._flush();
	_List.bindItems(table, column, where, onclick);
}

List.bindItemSubtitles = function(table, columns, where, orderby, onclick) {
	List._flush();
	_List.bindItemSubtitles(table, columns, where, orderby, onclick);
}
	
List.getValue = function(itemId) {
   return _List.getValue(itemId);
}

List.setEmpty = function(text) {
	_List.setEmpty(text);
}

List.show = function(style) {
	List._flush();
   _List.show(style);
}

///////////////////////

function History() {}
History.back = function() {_History.back();}
History.next = function() {_History.next();}
History.previous = function() {_History.previous();}
History.reload = function(funcCall) {_History.reload(funcCall);}
History.replace = function(funcCall) {_History.replace(funcCall);eval(funcCall);}


//////////////////////

function Calendar() {}
Calendar.setCallback = function(callback) {
	_List.setCallback(callback);
}

Calendar.addItem = function(title, date) {
	List.addItemLabel(title, String(date));
}

Calendar.show = function() {
	List.show("calendar");
}
