Config.appid = "sony.store";
Config.title = "Explore By Sony";
Config.version = "5";

Config.tables["products"] = "id;name;code;categoryid;shortdescription;description;thumbnail;price REAL;currency;rating INTEGER;popular INTEGER;parentid;seriesid;colors;promotionid;introdate DATE;url";
Config.tables["productimages"] = "id;productid;rank INTEGER;imgurl";
Config.tables["categories"] = "id;name;rank INTEGER;imgurl;thumbnail;parentid;haschild INTEGER;description";
Config.tables["groups"] = "id;name;rank INTEGER;thumbnail";
Config.tables["priceranges"] = "id;min INTEGER;max INTEGER;categoryid";
Config.tables["promotions"] = "id;name;description;startdate DATE;enddate DATE;imgurl;url;type INTEGER";
Config.tables["reviews"] = "id;productid;name;author;description;date DATE;rating INTEGER";
Config.tables["stores"] = "id;name;type;address;phone;businesshours;geo;zoneid;cityid;promourl;startdate DATE;enddate DATE;categoryid";
Config.tables["zones"] = "id;name";
Config.tables["cities"] = "id;name;zoneid";
Config.tables["polls"] = "id;name;date DATE;type INTEGER";
Config.tables["pollchoices"] = "id;pollid;name;votecount INTEGER";
Config.tables["videos"] = "id;url;name;categoryid";
Config.tables["videocategories"] = "id;name";

Config.tables["assets"] = "id;name;imgurl;description;groupid;url;productid;rank INTEGER";
Config.tables["assetgroups"] = "id;name;imgurl;categoryid;description;rank INTEGER";

/////////////////////////////////////////////////

function main() {
    // set the current country
    setCountry(Settings.get("country"));
    
    _orderByLabel = R.LATEST;

    if (Settings.getPlatform() == "mobileweb") {
        History.replace("mainMobileWeb()");
        return;
    }

    // custom sidebar items on native app
    addSideBarItems();
    
    var promos = Query.select("promotions", "id;imgurl", "type=2 AND startdate<={today()} AND enddate>={today()}", "startdate");
    var homePromo = (promos.length > 0) ? promos[0] : null;
    var products = getHomePageProducts(homePromo != null ? homePromo.id : null);
    var welcome = "Sony " + getCountryName();
    Toolbar.setTitle("");
    if (promos.length > 0) List.addImage(promos[0].imgurl, "viewPromotion({promos[0].id})");
    List.addItemLabel(welcome, Format.text(R.SEARCH));
    for (var i = 0; i < products.length; i++) {
        _writeProduct(products[i]);
    }

    List.addItem(R.MOREPRODUCTS, "viewProducts()", "class=gray");

    Toolbar.addTab(R.PRODUCTS2, "viewProducts()");
    Toolbar.addTab(R.PROMOTIONS, "viewPromotions()");
    Toolbar.addTab(R.STORES, "viewStoreLocator()");
    Toolbar.addButton("Search", "search('')", "search");
    List.addHeader(R.MORE);
    List.addItem(R.SUPPORT, "viewSupport()");
    List.addItem(R.VIDEOS, "viewVideos()");
    List.addItem(R.NEWS, "viewNews()");
    List.addItem(R.TECHNOLOGY, "viewTechnologies()");
    //List.addItem(R.WISHLIST, "viewCart()");
    //List.addItem(R.CHANGECOUNTRY, "viewCountries()");
    List.show();

    App.track("/home");
}

function customStartPage(hash) {
    if (hash == null || hash.length <= 3) return false;

    hash = hash.substring(1).toLowerCase(); // remove the #

    // 1. try category match
    var categories = Query.select("categories", "id;name");
    for (var i = 0; i < categories.length; i++) {
        var category = categories[i];
        var name = category.name.toLowerCase();
        if (name.indexOf(hash) > -1) {
            History.replace("viewCategory({category.id})");
            return true;
        }
    }
    // 2. Try with SKU match
    var products = Query.select("products", "id");
    for (var i = 0; i < products.length; i++) {
        var product = products[i];
        var productid = product.id.toLowerCase();
        if (productid.indexOf(hash) > -1) {
            History.replace("viewProduct({product.id})");
            return true;
        }
    }
    return false;
}

function mainMobileWeb() {
    if (customStartPage(CoreEngine.hash) == true) {
        return;
    }

    var appStoreLink = null;
    var ua = navigator.userAgent.toLowerCase();
    if ((ua.match(/iPhone/i)) || (ua.match(/iPod/i))) {
        appStoreLink = "http://itunes.apple.com/sg/app/sony-singapore/id442379296?mt=8";
    } else if (ua.indexOf("android") > -1) {
        appStoreLink = "https://market.android.com/details?id=Unyverse.sony";
    }

    var promos = Query.select("promotions", "id;imgurl", "type=2 AND startdate<={today()} AND enddate>={today()}", "startdate");
    var homePromo = (promos.length > 0) ? promos[0]: null;
    var products = getHomePageProducts(homePromo != null ? homePromo.id : null);

    var welcome = R.WELCOME + " " + getCountryName();
    Toolbar.setTitle(welcome);
    if (homePromo != null) List.addImage(homePromo.imgurl, "viewPromotion({homePromo.id})");
    List.addItemLabel(welcome, R.SEARCH);
    if (Settings.get("country") != "in" && appStoreLink != null) List.addItemLabel(R.GETFASTERACCESS, R.DOWNLOADEXPLORE, "App.web({appStoreLink})");
    
    for (var i = 0; i < products.length; i++) {
        _writeProduct(products[i]);
    }
    List.addItem(R.MOREPRODUCTS, "viewProducts()", "class=gray");

    Toolbar.addButton(R.VIDEOS, "viewVideos()", "sony_video");
    Toolbar.addButton(R.NEWS, "viewNews()", "sony_news");
    Toolbar.addButton(R.SUPPORT, "viewSupport()", "sony_support");
    Toolbar.addButton(R.TECHNOLOGY, "viewTechnologies()", "sony_techno");
    Toolbar.addButton(R.FULLSITE, "fullSite()", "sony_site");
    Toolbar.addButton(R.CHANGECOUNTRY, "viewCountries()", "sony_country");
    List.show();

    App.track("/home");
}

function getHomePageProducts(homePromoId) {
    var list = new Array();
    
    // First try the Home Page promos
    if (homePromoId != null) {
        list = Query.select("products", "id;code;name;thumbnail;price;currency;introdate", "promotionid CONTAINS {homePromoId}", "introdate DESC");
    }
    if (list.length >= 5) return list.slice(0,5);

    // Then any active promos
    var promos = Query.select("promotions", "id", "type=0 AND startdate<={today()} AND enddate>={today()}", "startdate DESC");
    var gifts = Query.select("promotions", "id", "type=3");
    promos = promos.concat(gifts);
    for (var i = 0; i < promos.length; i++) {
        var promoid = promos[i].id;
        var products = Query.select("products", "id;code;name;thumbnail;price;currency;introdate", "promotionid CONTAINS {promoid}");
        for (var j = 0; j < products.length; j++) {
            list.push(products[j]);
            if (list.length >= 5) { 
                return list;
            }
        }
    }
    
    // Then newest products
    var newest = Query.select("products", "id;code;name;thumbnail;price;currency;introdate", "parentid='' AND introdate<{today()}", "introdate DESC");
    for (var j = 0; j < newest.length; j++) {
        list.push(newest[j]);
        if (list.length >= 5) {
            return list;
        }
    }
    return list;
}


////////////// SEARCH

function search(str) {
    if (Settings.getPlatform() == "mobileweb") {
        if (HistoryManager.current().funcCall == "viewStoreLocator()") {
            onStoreSearch(str);
            return;
        }
    }

    if (str != null && str.length > 0) {
        var count = 0;
        count += writeProdutSearchSection(str, "parentid=''", R.PRODUCTS2);
        count += writeProdutSearchSection(str, "parentid!=''", R.ACCESSORIES);
        count += writeStoreSearchSection(str, "type!=3", R.STORES); // not service center
        // TODO : News & Videos
        if (count == 0) List.addItem(R.NORESULTS);
    }
    Toolbar.setSearch("Search Anything", str, "search");
    List.show();
}

function writeProdutSearchSection(search, where, title) {
    var products = Query.search("products", search, "name;code;shortdescription", where);
    var count = Math.min(products.length, 10);
    if (count > 0) List.addHeader(title + " (" + count + ")");
    for (var i = 0; i < count; i++) {
        var product = products[i];
        if (Settings.getPlatform() != "mobileweb") {
            // native app Query.search returns only "id, firstcolumn"
            product = Query.selectId("products", products[i].id);
        }
        _writeProduct(product);
    }
    return count;
}

//////////////////////////////
var _orderBy = "introdate DESC";
var _orderByLabel; // initialized in main

function viewProducts() {
    Toolbar.setTitle(R.PRODUCTS2);
    Toolbar.addTab(R.PRODUCTS2, "");
    Toolbar.addTab(R.PROMOTIONS, "viewPromotions()");
    Toolbar.addTab(R.STORES, "viewStoreLocator()");

    var groups = Query.select("groups", "id;name;thumbnail", null, "rank");
    for (var i = 0; i < groups.length; i++) {
        var group = groups[i];
        List.addItem(group.name, "viewGroup({group.id})", "class=category;img=" + group.thumbnail);
    }
    List.show();
    App.track("/products");
}

function viewGroup(groupid) {
    var visibleCategories = new Array();

    var group = Query.selectId("groups", groupid);
    var categories = Query.select("categories", "id;name;thumbnail", "parentid={groupid}", "rank");
    for (var i = 0; i < categories.length; i++) {
        var category = categories[i];
        var count = Query.count("products", "parentid='' AND categoryid={category.id}");
        if (count > 0) visibleCategories.push(category);
    }

    if (visibleCategories.length == 1) {
        viewCategory(visibleCategories[0].id);
        return;
    }

    for (var i = 0; i < visibleCategories.length; i++) {
        var category = visibleCategories[i];
        List.addItem(category.name, "viewCategory({category.id})", "class=category;img=" + category.thumbnail);
    }
    List.show();
}

function viewCategory(id) {
    var category = Query.selectId("categories", id);
    if (category.imgurl != '') List.addImage(category.imgurl, null, "height=100px");
    //if (category.description != '') List.addItem(R.WHYSONY, "viewWhySony({category.id})");

    var where = "categoryid={id} AND parentid=''";
    if (Settings.getPlatform() == "mobileweb") {
        List.addButton(R.SORTBY, "showSortBy()");
    } else {
        Toolbar.addButton(R.SORTBY, "showSortBy()");
    }
    var galleryCount = Query.count("assetgroups", "categoryid CONTAINS {id}");
    if (galleryCount > 0) List.addButton(R.SAMPLEPHOTOS, "viewAssetGroup({id})", "icon=arrow;class=black");
    Toolbar.setStyle("search");

    _computeAccessories();
    var accessories = _accessories[id];
    if (accessories != null) List.addButton(R.ACCESSORIES, "showAccessories({category.id})");
    List.addHeader(category.name + " - " + _orderByLabel);
    _writeProductList(where);
    List.show();
    App.track("/productcategory/" + encodeURIComponent(category.name));
}

var _accessories = null; // map of accessories by categoryid

function _computeAccessories() {
    // already computed?
    if (_accessories != null) return;
    _accessories = new Array();

    // 1. Create a product category Hashtable
    var productCategoryMap = new Array();
    var products = Query.select("products", "id;categoryid", "parentid=''");
    for (var i = 0; i < products.length; i++) {
        var product = products[i];
        productCategoryMap[product.id] = product.categoryid;
    }

    // for each accessory, get its first parent categoryid and add it to the list if is it the requested category
    var accessories = Query.select("products", "id;name;thumbnail;price;currency;code;parentid", "parentid!=''", "introdate DESC");
    for (var i = 0; i < accessories.length; i++) {
        var accessory = accessories[i];

        var accessoryCategoryMap = new Array();
        var parents = accessory.parentid.split("|");
        for (var j = 0; j < parents.length; j++) {
            var parentid = parents[j];
            var parentCategoryid = productCategoryMap[parentid];
            if (accessoryCategoryMap[parentCategoryid] == null) {
                if (_accessories[parentCategoryid] == null) _accessories[parentCategoryid] = new Array();
                _accessories[parentCategoryid].push(accessory);

                accessoryCategoryMap[parentCategoryid] = "1";
            }
        }
    }
}

function showAccessories(categoryid) {
    _computeAccessories();

    Toolbar.setStyle("search");
    var items = _accessories[categoryid];
    for (var i = 0; i < items.length; i++) {
        _writeProduct(items[i]);
    }
    List.show();
}

function showSortBy() {
    var country = Settings.get("country");
    List.addItem(R.LATEST, "onLatest()");
    if (country == "sg" || country == "au") List.addItem(R.POPULAR, "onPopular()");
    List.addItem(R.PRICE, "onPrice()");
    List.show("popup");
}

function onLatest() {
    _orderBy = "introdate DESC";
    _orderByLabel = R.LATEST;
    History.reload();
}

function onPopular() {
    _orderBy = "rating DESC";
    _orderByLabel = R.POPULAR;
    History.reload();
}

function onPrice() {
    _orderBy = "price";
    _orderByLabel = R.PRICE;
    History.reload();
}

function _writeProductList(where, header) {
    var products = Query.select("products", "id;name;thumbnail;price;currency;code;introdate", where, _orderBy);
    if (products.length == 0) return;
    if (header != null) List.addHeader(header);
    var count = products.length;
    for (var i = 0; i < count; i++) {
        _writeProduct(products[i]);
    }
}

function _writeProduct(product) {
    var subtitle = product.name;
    if (product.price > 0 && product.introdate <= today()) {
        var price = Format.price(product.price, product.currency);
        if (Settings.getPlatform() == "mobileweb") subtitle += " <div class=price>" + price + "</div>";
        else subtitle += " " + price;
    }
    List.addItemSubtitle(product.code, subtitle, "viewProduct({product.id})", "img=" + product.thumbnail);
}

function viewProduct(id) {
    var product = Query.selectId("products", id);
    var firstProductImage = Query.select("productimages", "imgurl", "productid={id} AND rank=1");
    var reviewCount = Query.count("reviews", "productid={id}");
    var galleryCount = Query.count("assetgroups", "categoryid CONTAINS {product.categoryid}");
    var promo = _getProductPromotion(product.promotionid);

    Toolbar.setTitle(R.PRODDETAILS);
    if (Settings.getPlatform() != "mobileweb") {
        Toolbar.addButton("Share", "App.share({product.url})");
        if (Wishlist.contains(product.id) == false) {
            Toolbar.addButton("Add to Wishlist", "Wishlist.add({product.id})");
        } else {
            Toolbar.addButton("Remove from Wishlist", "Wishlist.remove({product.id})");
        }
    }
    if (firstProductImage.length == 1) List.addImage(firstProductImage[0].imgurl, "viewGallery({id})", "height=180px;class=productimage");

    var productUrl = product.url.toLowerCase(); // Facebook & Twitter are Case Sensitive URL...
    List.addItemLabel(product.code, product.name, null, "class=productcode");
    var price = "";
    if (product.price > 0 && product.introdate <= today()) price = Format.price(product.price, product.currency);
    List.addItemRating(price, product.rating, null, "class=productrating");
    if (product.colors != '') List.addItemColor(R.AVAILABLEIN, product.colors);

    var description = product.shortdescription;
    List.addItemLabel("", description, null, "class=productdesc");

    if (Settings.getPlatform() == "mobileweb") List.addItemShare(productUrl);
    if (Settings.getPlatform() == "mobileweb") List.addItemLabel("", "<div class=pricedisclaimer>" + R.PRICE_DISCLAIMER + "</div>");

    List.addItem(R.SPECS, "viewSpec({id})", "icon=arrow;class=black");
    List.addItem(R.PHOTOGALLERY, "viewGallery({id})", "icon=arrow;class=black");
    if (galleryCount > 0) List.addItem(R.SAMPLEPHOTOS, "viewAssetGroup({product.categoryid})", "icon=arrow;class=black");
    if (promo != null) List.addItem(R.PROMOTION, "viewPromotion({promo.id})", "icon=arrow;class=black");
    if (reviewCount > 0) List.addItem(R.REVIEWS, "viewReviews({id})", "icon=arrow;class=black");
    if (product.parentid != '') {
        List.addItem(R.COMPATIBILITY, "viewCompatibility({product.parentid})", "icon=arrow;class=black");
    } else {
        var accessoriesCount = Query.count("products", "parentid CONTAINS {id}");
        if (accessoriesCount > 0) List.addItem(R.ACCESSORIES, "viewAccessories({id})", "icon=arrow;class=black");
    }
    List.addItem(R.WHERETOBUY, "viewStoreLocator()", "icon=arrow;class=black");

    var country = Settings.get("country");
    buyLabel = (country == "au" || country == "nz") ? "Buy Now" : "View in Full Site";
    List.addItem(buyLabel, "App.web({productUrl})", "icon=arrow;class=black");

    List.show();
    App.track("/product/" + encodeURIComponent(product.code));
}

function _getProductPromotion(promotionid) {
    if (promotionid == '') return null;
    var parts = promotionid.split("|");
    for (var i = 0; i < parts.length; i++) {
        var promo = Query.selectId("promotions", parts[i]);
        if (promo != null) {
            if (promo.type == 3 || (promo.type == 0 && promo.startdate <= today() && promo.enddate >= today())) {
                return promo;
            }
        }
    }
    return null;
}

function viewAccessories(productid) {
    var where = "parentid CONTAINS {productid}";
    _writeProductList(where);
    List.show();
}

function viewCompatibility(parentid) {
    var where = "id IN " + list(parentid);
    _writeProductList(where);
    List.show();
}

function viewGallery(productid) {
    var images = Query.select("productimages", "imgurl", "productid={productid}", "rank");
    var buf = [];
    for (var i = 0; i < images.length; i++) {
        buf.push(images[i].imgurl);
    }

    WebView.showImage(buf.join("|"));
}

///// Reviews
function viewReviews(productid) {
    var reviews = Query.select("reviews", "id;name;author;date;rating;description", "productid={productid}", "date DESC");
    for (var i = 0; i < reviews.length; i++) {
        var review = reviews[i];
        var header = review.name;
        if (header.length > 0) header += " - ";
        header += Format.date(review.date);
        List.addHeader(header);
        List.addItemLabel(" " + R.BY + " " + review.author, review.description);
        List.addItemRating("", review.rating);
    }
    List.show();
}

///////////////////////

function viewVideos() {
    Toolbar.setTitle(R.VIDEOS);
    var categories = Query.select("videocategories", "id;name", null, "name");
    for (var i = 0; i < categories.length; i++) {
        var category = categories[i];
        List.addItem(category.name, "viewVideo({category.id})");
    }
    List.show();
    App.track("/videos");
}

function viewVideo(categoryid) {
    var videos = Query.select("videos", "name;url", "categoryid={categoryid}");
    for (var i = 0; i < videos.length; i++) {
        var video = videos[i];
        var img = "http://i3.ytimg.com/vi/" + video.url + "/default.jpg";
        List.addItem(video.name, "App.video({video.url})", "img=" + img);
    }
    List.show();
}

function viewNews() {
    Toolbar.setTitle(R.NEWS);
    var news = Query.select("promotions", "id;name;startdate;imgurl", "type=4", "startdate DESC");
    for (var i = 0; i < news.length; i++) {
        var item = news[i];
        List.addItemSubtitle(item.name, Format.date(item.startdate), "viewNewsDetail({item.id})", "img=" + item.imgurl);
    }
    List.show();
    App.track("/news");
}

function viewNewsDetail(id) {
    var news = Query.selectId("promotions", id);
    Toolbar.setTitle(R.NEWS);
    Toolbar.setStyle("nextprevious");
    List.addImage(news.imgurl);
    List.addItem(news.name);
    var label = "";
    if (news.startdate > 0) label = Format.date(news.startdate);
    List.addItemLabel(label, news.description);
    if (news.url != '') List.addItem(R.MOREINFO, "App.web({news.url})", "icon=bluearrow");
    var where = "promotionid CONTAINS {id}";
    _writeProductList(where, R.APPLICABLEPRODUCTS);
    List.show();
}

function viewSpec(productid) {
    var baseUrl = "http://www.upvise.com/uws/user/";
    var specs = Query.rss(baseUrl + "sony.store.specs?productid=" + encodeURIComponent(productid));
    for (var i = 0; i < specs.items.length; i++) {
        var spec = specs.items[i];
        if (spec.type == 0) List.addHeader(spec.label);
        else List.addItemLabel(spec.label, spec.value);
    }
    Toolbar.setStyle("search");
    List.show();
}

function viewTechnologies() {
    Toolbar.setTitle(R.TECHNOLOGY);
    var items = Query.select("promotions", "id;name;description;imgurl", "type=1");
    for (var i = 0; i < items.length; i++) {
        var item = items[i];
        List.addItemSubtitle(item.name, Format.text(item.description), "viewTechnology({item.id})", "img=" + item.imgurl);
    }
    List.show();
    App.track("/technology");
}

function viewTechnology(id) {
    var promo = Query.selectId("promotions", id);
    Toolbar.setTitle(R.TECHNOLOGYDETAILS);
    Toolbar.setStyle("nextprevious");
    if (promo.imgurl != '') List.addImage(promo.imgurl);
    List.addItemLabel("", promo.description);
    var where = "promotionid CONTAINS {id}";
    _writeProductList(where, R.PRODUCTS2);
    List.show();
}

////////////////////////////// Promotions

function viewPromotions() {
    Toolbar.setTitle(R.PROMOTIONS);
    Toolbar.addTab(R.PRODUCTS2, "viewProducts()");
    Toolbar.addTab(R.PROMOTIONS, "");
    Toolbar.addTab(R.STORES, "viewStoreLocator()");

    var promos = Query.select("promotions", "id;name;imgurl;startdate;enddate", "type=0 AND startdate<={today()} AND enddate>={today()}", "startdate DESC");
    var stores = Query.select("stores", "id;name;promourl", "promourl!='' AND startdate<={today()} AND enddate>={today()}", "startdate DESC");
    var gifts = Query.select("promotions", "id;name;imgurl", "type=3");
    for (var i = 0; i < promos.length; i++) {
        var promo = promos[i];
        var subtitle = Format.date(promo.startdate) + " - " + Format.date(promo.enddate);
        List.addItemSubtitle(promo.name, subtitle, "viewPromotion({promo.id})", "img=" + promo.imgurl);
    }
    List.addHeader(R.STOREPROMO);
    for (var i = 0; i < stores.length; i++) {
        var store = stores[i];
        var subtitle = Format.date(store.startdate) + " - " + store.date(promo.enddate);
        List.addItemSubtitle(store.name, subtitle, "viewStore({store.id})", "img=" + store.promourl);
    }
    List.addHeader(R.GIFTIDEAS);
    for (var i = 0; i < gifts.length; i++) {
        var gift = gifts[i];
        List.addItem(gift.name, "viewPromotion({gift.id})", "img=" + gift.imgurl);
    }
    List.show();
    App.track("/promotions");
}

function viewPromotion(id) {
    var promo = Query.selectId("promotions", id);
    Toolbar.setTitle(R.PROMODETAILS);
    if (promo.imgurl != '') List.addImage(promo.imgurl);
    var subtitle = "";
    if (promo.type == 0) subtitle = Format.date(promo.startdate) + " - " + Format.date(promo.enddate);
    List.addItemSubtitle(promo.name, subtitle);
    List.addItemLabel("", promo.description);

    _writeProductList("promotionid CONTAINS {id}", R.APPLICABLEPRODUCTS);
    if (promo.url != '') List.addItem(R.MOREINFO, "App.web({promo.url})", "class=black");
    List.show();
    App.track("/promotion/" + encodeURIComponent(promo.name));
}

////////// Store Locator

function viewStoreLocator() {
    Toolbar.addTab(R.PRODUCTS2, "viewProducts()");
    Toolbar.addTab(R.PROMOTIONS, "viewPromotions()");
    Toolbar.addTab(R.STORES, "");

    var count = Query.count("stores", "type<3 AND categoryid!=''");
    Toolbar.setTitle(R.STORELOCATOR);
    List.addButton(R.SEARCHNAME, "searchStore()");
    var label = Settings.get("country") == "sg" ? R.SEARCHZONE : R.SEARCHSTATE;
    var where = 'type<3'; // stores, not service center
    List.addButton(label, "viewZoneList({where})");
    if (Settings.getPlatform() != "mobileweb") List.addButton(R.NEARESTSTORES, "viewNearestStoreList({where})");
    if (count > 0) List.addButton(R.BYPRODUCTCATEGORY, "viewProductCategoryStoreList({where})");
    List.show();
    App.track("/storelocator");
}

function searchStore() {
    if (Settings.getPlatform() == "mobileweb") {
         _searchStoreOnly = true;
        showSearch();
    } else {
        var str = App.prompt(R.ENTERSTORE, '');
        if (str == null) History.reload();
        else History.replace("onStoreSearch({str})");
    }
}

function onStoreSearch(str) {
    writeStoreSearchSection(str, "type!=3", R.STORES); // not service center
    List.show();
}

function writeStoreSearchSection(search, where, title) {
    var stores = Query.search("stores", search, "name;address", where);
    // TODO sort by type ascending for Sony Store first....
    var count = Math.min(stores.length, 20);
    if (count > 0) List.addHeader(title + " (" + count + ")");
    for (var i = 0; i < count; i++) {
        var store = stores[i];
        if (Settings.getPlatform() != "mobileweb") store = Query.selectId("stores", store.id);
        List.addItemSubtitle(store.name, store.address, "viewStore({store.id})")
    }
    return count;
}

function viewStoreList(where) {
    if (where.indexOf("type=3") != -1) {
        var serviceShops = Query.select("stores", "id;name;address;promourl", where, "name");
        for (var i = 0; i < serviceShops.length; i++) {
            var shop = serviceShops[i];
            List.addItemSubtitle(shop.name, shop.address, "viewStore({shop.id})");
        }
    } else {
        var sonyShops = Query.select("stores", "id;name;address;promourl", "type<2 AND " + where, "name");
        var otherShops = Query.select("stores", "id;name;address;promourl", "type=2 AND " + where, "name");
        if (sonyShops.length > 0) List.addHeader(R.SONYSHOPS);
        for (var i = 0; i < sonyShops.length; i++) {
            var shop = sonyShops[i];
            List.addItemSubtitle(shop.name, shop.address, "viewStore({shop.id})");
        }
        if (otherShops.length > 0) List.addHeader(R.DEALERS);
        for (var i = 0; i < otherShops.length; i++) {
            var shop = otherShops[i];
            List.addItemSubtitle(shop.name, shop.address, "viewStore({shop.id})");
        }
    }
    List.show();
}

function viewZoneList(where) {
    var zones = Query.select("zones", "id;name", null, "name");
    var citiesCount = Query.count("cities");

    var shops = Query.select("stores", "zoneid", where);
    var counter = _groupcount(shops, "zoneid");
    for (var i = 0; i < zones.length; i++) {
        var zone = zones[i];
        var where2 = where + " AND zoneid={zone.id}";
        var onclick = citiesCount > 0 ? "viewCityList({where},{zone.id})" : "viewStoreList({where2})";
        var count = counter[zone.id];
        if (count != null) List.addItem(zone.name, onclick, "number=" + count);
    }
    List.show();
}

function viewCityList(where, zoneid) {
    var cities = Query.select("cities", "id;name", "zoneid={zoneid}", "name");
    var shops = Query.select("stores", "cityid", where);
    var counter = _groupcount(shops, "cityid");
    for (var i = 0; i < cities.length; i++) {
        var city = cities[i];
        var count = counter[city.id];
        var where2 = where + " AND cityid={city.id}";
        if (count > 0) List.addItem(city.name, "viewStoreList({where2})", "number=" + count);
    }

    // also display all stores for zoneid where city=''
    var shops = Query.select("stores", "id;name;address", where + " AND zoneid={zoneid} AND cityid=''", "name");
    for (var i = 0; i < shops.length; i++) {
        var shop = shops[i];
        List.addItemSubtitle(shop.name, shop.address, "viewStore({shop.id})");
    }

    List.show();
}

function viewNearestStoreList(where) {
    var stores = Query.select("stores", "id;name;geo", where + " AND geo!=''");
    for (var i = 0; i < stores.length; i++) {
        var store = stores[i];
        store.distance = Settings.getDistanceTo(store.geo);
    }
    stores.sort(function (o1, o2) {
        return o1.distance - o2.distance;
    });

    var count = Math.min(20, stores.length);
    for (var i = 0; i < count; i++) {
        var store = stores[i];
        //if (item.distance > 100 * 1000) break; // no more than 100km
        List.addItemSubtitle(store.name, Format.distance(store.distance), "viewStore({store.id})");
    }
    List.show();
}

function viewProductCategoryStoreList(where) {
    var categories = Query.select("categories", "id;mame", null, "name");
    for (var i = 0; i < categories.length; i++) {
        var category = categories[i];
        var where2 = where + " AND categoryid CONTAINS {category.id}";
        var count = Query.count("stores", where2);
        if (count > 0) List.addItem(category.name, "viewZoneList({where2})");
    }
    List.show();
}

function viewStore(id) {
    var store = Query.selectId("stores", id);
    var title = (store.type == 3) ? R.SERVICECENTERDETAILS : R.STOREDETAILS;
    if (store.promourl != '' && store.startdate <= today() && store.enddate >= today()) List.addImage(store.promourl);
    List.addItem(store.name);
    if (store.promourl != '') List.addItemLabel(R.PROMODATE, Format.date(store.startdate) + " " + R.TO + " " + Format.date(store.enddate));
    List.addItemLabel(R.HOURS, store.businesshours);
    List.addItemLabel(R.ADDRESS, store.address);
    List.addButton(R.VIEWMAP, "App.map({store.name},{store.geo})");
    List.addButton(R.CALLNOW, "App.call({store.phone})");

    if (store.categoryid != null && store.categoryid != "") {
        List.addHeader(R.PRODUCTCATEGORIES);
        var categories = Query.select("categories", "id;name;thumbnail", "id IN " + list(store.categoryid), "rank");
        for (var i = 0; i < categories.length; i++) {
            var category = categories[i];
            var count = Query.count("products", "parentid='' AND categoryid={category.id}");
            if (count > 0) List.addItem(category.name, "viewCategory({category.id})", "class=category;img=" + category.thumbnail);
        }
    }
    List.show();
}

//////////////////// Support
function viewSupport() {
    Toolbar.setTitle(R.SUPPORT);

    if (Settings.get("country") == "sg") {
        var imgurl = "http://www.sony.com.sg/microsite/support/images/1280x300_mobile.jpg";
        var onclick = "App.web('https://www.sony.com.sg/microsite/MySony/faqs/mysony_intro/whatis.html')";
        List.addImage(imgurl, onclick);
    }

    var number = Settings.get("supportphone");
    if (number != "") {
        List.addHeader(R.CUSTINFOCENTER);
        if (Settings.get("country") == "in") {
            List.addButton(R.CALLSONY + " (" + R.MOBILEPRODUCTS + ")", "App.call('1800 3000 2800')");
            List.addButton(R.CALLSONY + " (" + R.OTHERPRODUCTS + ")", "App.call({number})");
        } else {
            List.addButton(R.CALLSONY, "App.call({number})");
        }
    }

    List.addHeader(R.SERVICECENTERS);
    var where = "type=3"; // service center
    var count = Query.count("stores", "type=3");
    if (count < 10) {
        viewStoreList(where);
    } else {
        var label = Settings.get("country") == "sg" ? R.SEARCHZONE : R.SEARCHSTATE;
        List.addButton(label, "viewZoneList({where})");
        if (Settings.getPlatform() != "mobileweb") List.addButton(R.NEARESTSERVICECENTERS, "viewNearestStoreList({where})");
        var count = Query.count("stores", "type=3 AND categoryid!=''");
        if (count > 0) List.addButton(R.BYPRODUCTCATEGORY, "viewProductCategoryStoreList({where})");
        List.show();
    }
    App.track("/support");
}

////////////////////////////
function Wishlist() {}

Wishlist._get = function() {
    var value = Settings.get("wishlist");
    return (value != null) ? value.split("|") : [];
}

Wishlist.contains = function (id) {
    var ids = Wishlist._get();
    for (var i = 0; i < ids.length; i++) {
        if (id == ids[i]) return true;
    }
    return false;
}

Wishlist.add = function (id) {
    var ids = Wishlist._get();
    ids.push(id);
    Settings.set("wishlist", ids.join("|"));
    History.reload();
}

Wishlist.remove = function (id) {
    var ids = Wishlist._get();
    var newids = [];
    for (var i = 0; i < ids.length; i++) {
        if (ids[i] != id) newids.push(ids[i]);
    }
    Settings.set("wishlist", newids.join("|"));
    History.reload();
}

Wishlist.clear = function() {
    Settings.set("wishlist", "");
    History.reload();
}

Wishlist.viewList = function () {
    Toolbar.setTitle("Wishlist");
    Toolbar.addButton("Clear", "Wishlist.clear()");
    var ids = Wishlist._get();
    for (var i = 0; i < ids.length; i++) {
        var product = Query.selectId("products", ids[i]);
        if (product != null) {
            var subtitle = product.name;
            if (product.price > 0 && product.introdate <= today()) {
                var price = Format.price(product.price, product.currency);
                subtitle += " " + price;
            }
            var style = "img=" + product.thumbnail + "icon=delete;onicon=";
            List.addItemSubtitle(product.code, subtitle, "viewProduct({product.id})", style);
        }
    }
    List.show();
}

//////////////////////////////
function viewAssetGroup(categoryid) {
    var galleries = Query.select("assetgroups", "id;name;imgurl", "categoryid CONTAINS {categoryid}", "name");
    for (var i = 0; i < galleries.length; i++) {
        var gallery = galleries[i];
        List.addItem(gallery.name, "viewAssetList({gallery.id})", "img=" + gallery.imgurl);
    }
    List.show();
}

function viewAssetList(groupid) {
    var photos = Query.select("assets", "id;name;imgurl;description", "groupid={groupid}", "name");
    for (var i = 0; i < photos.length; i++) {
        var photo = photos[i];
        List.addItem(photo.name, "viewAsset({photo.id})", "img=" + photo.imgurl);
    }
    List.show();
}

function viewAsset(id) {
    var photo = Query.selectId("assets", id);
    Toolbar.setStyle("nextprevious");
    List.addImage(photo.imgurl);
    List.addItemLabel("", photo.description);
    if (photo.url != "") List.addItem(R.MOREINFO, "App.web({photo.url})", "class=black");
    if (photo.productid != '') List.addItem(R.PRODUCTS2, "viewAssetProducts({photo.productid})", "class=black");
    List.show();
}

function viewAssetProducts(productid) {
    var where = "id IN " + list(productid);
    _writeProductList(where);
    List.show();
}

////////////////////////////

function termsOfUse() {
    window.open(_fullSiteUrl + "/section/termsofuse", "_blank");
}

function fullSite() {
    window.open(_fullSiteUrl, "_blank");
}

function privacyPolicy() {
    window.open(_fullSiteUrl + "/section/privacypolicy", "_blank");
}

function viewCountries() {
    var url = "http://www.sony.com.sg/HP/images/flag/";
    List.addItem(R.AUSTRALIA, "setCountry('au')", "img=" + url + "AU_flag.png");
    if (Settings.getPlatform() == "mobileweb") List.addItem(R.INDIA, "setCountry('in')", "img=" + url + "IN_flag.png");
    List.addItem(R.MALAYSIA, "setCountry('my')", "img=" + url + "MY_flag.png");
    List.addItem(R.SINGAPORE, "setCountry('sg')", "img=" + url + "SG_flag.png");
    List.addItem(R.THAILAND, "setCountry('th')", "img=" + url + "TH_flag.png");
    List.addItem(R.VIETNAM, "setCountry('vn')", "img=" + url + "VN_flag.png");
        
    if (_TEST()) { // Testing
        List.addItem(R.INDONESIA, "setCountry('id')", "img=" + url + "ID_flag.png");
        List.addItem(R.NEWZEALAND, "setCountry('nz')", "img=" + url + "NZ_flag.png");
    }
    List.show();
}

var _fullSiteUrl;
 
function setCountry(country) {
    var supportPhone = "";
    var trackingId;
    var language = "en";
    var token;

    // try to get country from settings or from phone locale
    if (country == null) {
        country = Settings.getCountry();
    }

    if (country == 'in') {
        token = "2/v5YYtZVYRmsLv54ul/7D+g7KgUtIND"; // Sony IN
        supportPhone = "1800-103-7799";
        _fullSiteUrl = "http://www.sony.co.in";
        trackingId = "UA-31705795-1";
    } else if (country == "my") {
        token = "6ssYO8DfaknuAzq2JtDxNaX5yPE7HYSB0"; // Sony MY
        supportPhone = "1300 88 1233";
        _fullSiteUrl = "http://www.sony.com.my";
        trackingId = "UA-36184092-1";
    } else if (country == "au") {
        token = "AIirXMM8eWsX7PnI+OSBjqUuSWbmiYc1"; // Sony AU
        supportPhone = "1300 137 669";
        _fullSiteUrl = "http://www.sony.com.au";
        trackingId = "UA-36127284-1";
    } else if (country == "nz") {
        token = "ob8zVqAryah2mdI43zx44D-g7KgUtIND0"; // Sony NZ
        supportPhone = "0800 766 969";
        _fullSiteUrl = "http://www.sony.com.nz";
        trackingId = "UA-36127284-1";
    } else if (country == "th") {
        token = "inA/bcNdjsKt8Isc7ijonpz/mGeLmv1D"; // Sony TH
        supportPhone = "027156100";
        language = "th";
        _fullSiteUrl = "http://www.sony.co.th";
        trackingId = "UA-36128468-1";
    } else if (country == "vn") {
        token = "uPs3Sn2rLbfvrA4xBoKBd6UuSWbmiYc10"; // Sony VN
        supportPhone = "1800588885";
        language = "vi"; 
        _fullSiteUrl = "http://www.sony.com.vn";
        trackingId = "UA-36130586-1";
    } else if (country == "id") {
        token = "OJS19jHN42UYD4gftW6Hf8ha_Zz-H0BJ0"; // Sony Indonesia
        supportPhone = "021 2994 8800";
        _fullSiteUrl = "http://www.sony.co.id";
        trackingId = "UA-36622196-1";
    } else { // default is sg
        country = "sg";
        token = "ixo02aJTeKPO1wmWEhzffj+g7KgUtIND"; // Sony SG
        supportPhone = "6544 8600";
        _fullSiteUrl = "http://www.sony.com.sg";
        trackingId = "UA-33750921-1";
    }

    App.signInAuto(token, trackingId);
    Settings.set("supportphone", supportPhone);
    Settings.set("country", country);
    
    // set the web page site & header / footer
    if (Settings.getPlatform() == "mobileweb") {
        // Load strings for language
        _loadJS("sonystore." + language + ".js", false);

        document.title = "Sony " + getCountryName();
        _set("header", header());
        _set("footer", footer());
    }
}

function getCountryName() {
    var code = Settings.get("country");
    if (code == 'au') return R.AUSTRALIA;
    else if (code == 'id') return R.INDONESIA;
    else if (code == 'in') return R.INDIA;
    else if (code == 'nz') return R.NEWZEALAND;
    else if (code == 'my') return R.MALAYSIA;
    else if (code == 'sg') return R.SINGAPORE;
    else if (code == 'th') return R.THAILAND;
    else if (code == 'vn') return R.VIETNAM;
    else if (code == 'sg') return R.SINGAPORE;
    else return "";
}

function _TEST() {
    return (document.location.hostname.indexOf("m.sony.") == -1);
}

function header() {
    var onProducts = "CoreEngine.go('viewProducts()')";
    var onPromos = "CoreEngine.go('viewPromotions()')";
    var onStores = "CoreEngine.go('viewStoreLocator()')";
    var buf = [];
    //buf.push('<table><tr><td onclick="CoreEngine.goHome()" class=border><div class=sonylogo></div></td>');
    buf.push('<table><tr><td onclick="CoreEngine.goHome()" class=border><img height=32px src=sonybig.jpg /></td>');
    buf.push('<td onclick="', onProducts, '" class=border>', R.PRODUCTS2, '</td>');
    buf.push('<td onclick="', onPromos, '" class=border>', R.PROMOTIONS, '</td>');
    buf.push('<td onclick="', onStores, '" class=border>', R.STORES, '</td>');
    buf.push('<td onclick="showSearch()"><div class="tb_sony_search"></div></td></tr></table>');
    return buf.join('');
}

function footer() {
    var country = Settings.get("country").toUpperCase();
    var countryFlagUrl = "http://www.sony.com.sg/HP/images/flag/" + country + "_flag.png";
        
    var buf = [];
    buf.push('<span onclick="termsOfUse()">Terms of Use</span> <span style="float:right;" onclick="privacyPolicy()">Privacy Policy</span>');
    buf.push('<br /><br /><div style="text-align:right" id="flag"><img src="', countryFlagUrl, '" /></div>');
    return buf.join('');
}

////////////////////
function addSideBarItems() {
    Sidebar.clear();
    //Sidebar.addItem("Home", "sony.store", "main()");
    Sidebar.addItem(R.PRODUCTS2, "sony.store", "viewProducts()");
    Sidebar.addItem(R.PROMOTIONS, "sony.store", "viewPromotions()");
    Sidebar.addItem(R.STORES, "sony.store", "viewStoreLocator()");
    Sidebar.addItem(R.VIDEOS, "sony.store", "viewVideos()");
    Sidebar.addItem(R.NEWS, "sony.store", "viewNews()");
    Sidebar.addItem(R.TECHNOLOGY, "sony.store", "viewTechnologies()");
    Sidebar.addItem(R.SUPPORT, "sony.store", "viewSupport()");
    Sidebar.addItem(R.WISHLIST, "sony.store", "Wishlist.viewList()");
    Sidebar.addItem(R.CHANGECOUNTRY, "sony.store", "viewCountries()");
}